#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='cors',
    version='0.9.7',
    description='Fast CORS misconfiguration vulnerabilities scanner',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Jianjun Chen',
    author_email= 'whucjj@hotmail.com',
    url='http://github.com/chenjj/CORScanner',
    project_urls={  
        'Bug Reports': 'https://github.com/chenjj/CORScanner/issues',
        'Source': 'https://github.com/chenjj/CORScanner/',
    },
    license='MIT',
    packages=find_packages(),
    install_requires=['colorama', 'requests', 'argparse', 'gevent', 'tldextract', 'future'],
    include_package_data=True,
    zip_safe=False,
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Environment :: Console',
        'Topic :: Security',
    ],
    entry_points={
        'console_scripts': [
            'cors = CORScanner.cors_scan:main',
        ],
    },
)



#!/usr/bin/env python

import json
import sys
import argparse
import threading

from common.common import *
from common.logger import Log
from common.corscheck import CORSCheck

import gevent
from gevent import monkey
monkey.patch_all()
from gevent.pool import Pool
from gevent.queue import Queue
from colorama import init

# Globals
results = []

def banner():
    print(("""%s
   ____ ___  ____  ____   ____    _    _   _ _   _ _____ ____  
  / ___/ _ \|  _ \/ ___| / ___|  / \  | \ | | \ | | ____|  _ \ 
 | |  | | | | |_) \___ \| |     / _ \ |  \| |  \| |  _| | |_) |
 | |__| |_| |  _ < ___) | |___ / ___ \| |\  | |\  | |___|  _ < 
  \____\___/|_| \_\____/ \____/_/   \_\_| \_|_| \_|_____|_| \_\
                                                               %s%s
        # Coded By Jianjun Chen - whucjj@gmail.com%s
    """ % ('\033[91m', '\033[0m', '\033[93m', '\033[0m')))


def parser_error(errmsg):
    banner()
    print(("Usage: python " + sys.argv[0] + " [Options] use -h for help"))
    print(("Error: " + errmsg))
    sys.exit()


def parse_args():
    # parse the arguments
    parser = argparse.ArgumentParser(
        epilog='\tExample: \r\npython ' + sys.argv[0] + " -u google.com")
    parser.error = parser_error
    parser._optionals.title = "OPTIONS"
    parser.add_argument(
        '-u', '--url', help="URL/domain to check it's CORS policy")
    parser.add_argument(
        '-i',
        '--input',
        help='URL/domain list file to check their CORS policy')
    parser.add_argument(
        '-t',
        '--threads',
        help='Number of threads to use for CORS scan',
        type=int,
        default=50)
    parser.add_argument('-o', '--output', help='Save the results to json file')
    parser.add_argument(
        '-v',
        '--verbose',
        help='Enable Verbosity and display results in realtime',
        nargs='?',
        default=False)
    parser.add_argument('-d', '--headers', help='Add headers to the request.', default=None, nargs='*')
    args = parser.parse_args()
    if not (args.url or args.input):
        parser.error("No url inputed, please add -u or -i option")
    return args


# Synchronize results
c = threading.Condition()

def scan(cfg, log):
    global results

    while not cfg["queue"].empty():
        try:
            item = cfg["queue"].get(timeout=1.0)
            cors_check = CORSCheck(item, cfg)
            msg = cors_check.check_one_by_one()

            # Keeping results to be written to file only if needed
            if log.filename and msg:
                c.acquire()
                results.append(msg)
                c.release()
        except Exception as e:
            print(e)
            break

def cors_check(url, headers=None):
    # 0: 'DEBUG', 1: 'INFO', 2: 'WARNING', 3: 'ALERT', 4: 'disable log'
    log = Log(None, print_level=4)
    cfg = {"logger": log, "headers": headers}

    cors_check = CORSCheck(url, cfg)
    #msg = cors_check.check_all_in_parallel()
    msg = cors_check.check_one_by_one()
    return msg

def main():
    init()
    args = parse_args()
    banner()

    queue = Queue()
    log_level = 1 if args.verbose else 2  # 1: INFO, 2: WARNING

    log = Log(args.output, log_level)
    cfg = {"logger": log, "queue": queue, "headers": parse_headers(args.headers)}

    read_urls(args.url, args.input, queue)

    print("Starting CORS scan...")
    threads = [gevent.spawn(scan, cfg, log) for i in range(args.threads)]

    try:
        gevent.joinall(threads)
    except KeyboardInterrupt as e:
        pass

    # Writing results file if output file has been set
    if log.filename:
        with open(log.filename, 'w') as output_file:
            output_file.write(json.dumps(results, indent=4))
            output_file.close()
    print("Finished CORS scanning...")


if __name__ == '__main__':
    main()



import linecache


def normalize_url(i):
    if '://' in i:
        return [i]
    else:
        return ["http://" + i, "https://" + i]

def parse_headers(headers):
    if headers == None:
        return None
    else:
        parsedheaders = {}
    
        for header in headers:
            index = header.find(":")
            if index == -1:
                return None
            parsedheaders[header[0:index].strip()] = header[index+1:].strip()
        return parsedheaders

def read_file(input_file):
    lines = linecache.getlines(input_file)
    return lines


def read_urls(test_url, input_file, queue):
    if test_url:
        for u in normalize_url(test_url):
            queue.put(u)
    if input_file:
        lines = read_file(input_file)
        for i in lines:
            for u in normalize_url(i.strip()):
                queue.put(u)



import gevent.monkey
gevent.monkey.patch_all()

import requests, json, os, inspect, tldextract

from future.utils import iteritems
try:
    from urllib.parse import urlparse
except Exception as e:
    from urlparse import urlparse

import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

from threading import Thread

class CORSCheck:
    """docstring for CORSCheck"""
    url = None
    cfg = None
    headers = None
    result = {}

    def __init__(self, url, cfg):
        self.url = url
        self.cfg = cfg
        self.all_results = []
        if cfg["headers"] != None:
            self.headers = cfg["headers"]
        
    def send_req(self, url, origin):
        try:

            headers = {
                'Origin':
                origin,
                'Cache-Control':
                'no-cache',
                'User-Agent':
                'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36'
            }
            if self.headers != None:
                headers.update(self.headers)

            # self-signed cert OK, follow redirections
            resp = requests.get(self.url, timeout=5, headers=headers, verify=False, allow_redirects=True)

            # remove cross-domain redirections, which may cause false results
            first_domain =tldextract.extract(url).registered_domain
            last_domain = tldextract.extract(resp.url).registered_domain

            if(first_domain.lower() != last_domain.lower()):
                resp = None

        except Exception as e:
            resp = None
        return resp

    def get_resp_headers(self, resp):
        if resp == None:
            return None
        resp_headers = dict(
            (k.lower(), v) for k, v in iteritems(resp.headers))
        return resp_headers

    def check_cors_policy(self, test_module_name,test_origin,test_url):
        resp = self.send_req(self.url, test_origin)
        resp_headers = self.get_resp_headers(resp)
        status_code = resp.status_code if resp is not None else None

        if resp_headers == None:
            return None
        
        parsed = urlparse(str(resp_headers.get("access-control-allow-origin")))
        if test_origin != "null":
            resp_origin = parsed.scheme + "://" + parsed.netloc.split(':')[0]
        else:
            resp_origin = str(resp_headers.get("access-control-allow-origin"))

        msg = None

        # test_origin does not have to be case sensitive
        if test_origin.lower() == resp_origin.lower():
            credentials = "false"

            if resp_headers.get("access-control-allow-credentials") == "true":
                credentials = "true"
            
            # Set the msg
            msg = {
                "url": test_url,
                "type": test_module_name,
                "credentials": credentials,
                "origin": test_origin,
                "status_code" : status_code
            }
        return msg

    def is_cors_permissive(self,test_module_name,test_origin,test_url):
        msg = self.check_cors_policy(test_module_name,test_origin,test_url)

        if msg != None:
            self.cfg["logger"].warning(msg)
            self.result = msg
            self.all_results.append(msg)
            return True

        self.cfg["logger"].info("nothing found for {url: %s, origin: %s, type: %s}" % (test_url, test_origin, test_module_name))
        return False

    def test_reflect_origin(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        test_origin = parsed.scheme + "://" + "evil.com"

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)

    def test_prefix_match(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        test_origin = parsed.scheme + "://" + parsed.netloc.split(':')[0] + ".evil.com"

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_suffix_match(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        sld = tldextract.extract(test_url.strip()).registered_domain
        test_origin = parsed.scheme + "://" + "evil" + sld

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_trust_null(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        test_origin = "null"

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_include_match(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        sld = tldextract.extract(test_url.strip()).registered_domain
        test_origin = parsed.scheme + "://" + sld[1:]

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_not_escape_dot(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        sld = tldextract.extract(test_url.strip()).registered_domain
        domain = parsed.netloc.split(':')[0]
        test_origin = parsed.scheme + "://" + domain[::-1].replace(
            '.', 'a', 1)[::-1]

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_trust_any_subdomain(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        test_origin = parsed.scheme + "://" + "evil." + parsed.netloc.split(':')[0]

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_https_trust_http(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        if parsed.scheme != "https":
            return
        test_origin = "http://" + parsed.netloc.split(':')[0]

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        return self.is_cors_permissive(module_name,test_origin,test_url)


    def test_custom_third_parties(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        sld = tldextract.extract(test_url.strip()).registered_domain
        domain = parsed.netloc.split(':')[0]
        
        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        is_cors_perm = False

        # Opening origins file
        with open(os.path.join(os.path.dirname(os.path.realpath(__file__)),'..%sorigins.json' % os.sep)) as origins_file:  
            origins = json.load(origins_file)['origins']

            for test_origin in origins:

                is_cors_perm = self.is_cors_permissive(module_name,test_origin,test_url)
                if is_cors_perm: break

        return is_cors_perm
    
    def test_special_characters_bypass(self):
        module_name = inspect.stack()[0][3].replace('test_','');
        test_url = self.url
        parsed = urlparse(test_url)
        special_characters = ['_','-','"','{','}','+','^','%60','!','~','`',';','|','&',"'",'(',')','*',',','$','=','+',"%0b"]

        origins = []

        for char in special_characters:
            attempt = parsed.scheme + "://" + parsed.netloc.split(':')[0] + char + ".evil.com"
            origins.append(attempt)
            
        is_cors_perm = False

        self.cfg["logger"].info(
            "Start checking %s for %s" % (module_name,test_url))

        for test_origin in origins:
            is_cors_perm = self.is_cors_permissive(module_name,test_origin,test_url)
            if is_cors_perm: break

        return is_cors_perm

    def check_one_by_one(self):
        functions = [
            'test_reflect_origin',
            'test_prefix_match',
            'test_suffix_match',
            'test_trust_null',
            'test_include_match',
            'test_not_escape_dot',
            'test_custom_third_parties',
            'test_special_characters_bypass',
            'test_trust_any_subdomain',
            'test_https_trust_http',
        ]

        for fname in functions:
            func = getattr(self,fname)
            # Stop if we found a exploit case.
            if(func()): break 

        return self.result

    def check_all_in_parallel(self):
        functions = [
            'test_reflect_origin',
            'test_prefix_match',
            'test_suffix_match',
            'test_trust_null',
            'test_include_match',
            'test_not_escape_dot',
            'test_custom_third_parties',
            'test_special_characters_bypass',
            'test_trust_any_subdomain',
            'test_https_trust_http',
        ]

        threads = []
        for fname in functions:
            func = getattr(self,fname)
            t = Thread(target=func)
            t.start()
            threads.append(t)

        for t in threads:
            t.join()

        return self.all_results


import time
import json
import sys


class Log:
    """Class Log for logging CORS misconfiguration message"""
    print_level = 0
    msg_level = {0: 'DEBUG', 1: 'INFO', 2: 'WARNING', 3: 'ALERT'}
    auto_timestamp = 1

    def __init__(self, filename, print_level, auto_timestamp=1):
        self.filename = filename
        self.print_level = print_level
        self.auto_timestamp = auto_timestamp

    def write(self, msg, level=0, auto_timestamp=1):
        try:
            if level >= self.print_level:
                if self.auto_timestamp == 1:
                    timestamp = time.strftime("%Y-%m-%d %H:%M:%S",
                                              time.localtime())
                    record = "%s %s %s" % (timestamp, self.msg_level[level],
                                           msg)
                    sys.stdout.write(record + "\r\n")
                else:
                    sys.stdout.write(msg + "\r\n")
                sys.stdout.flush()
        except KeyboardInterrupt:
            self.close()

    def debug(self, msg):
        self.write(msg, 0)

    def info(self, msg):
        self.write(msg, 1)

    def warning(self, msg):
        record = "Found misconfiguration! " + json.dumps(msg)
        self.write("""%s%s%s""" % ('\033[91m', record, '\033[0m'), 2)

    def alert(self, msg):
        self.write(msg, 3)

    def close(self):
        if self.log:
            self.log.close()
